==> modularui-2.5.1-sources.jar: com/cleanroommc/modularui/screen/viewport/TransformationMatrix.java <==
package com.cleanroommc.modularui.screen.viewport;

import com.cleanroommc.modularui.api.layout.IViewport;
import com.cleanroommc.modularui.utils.Matrix4f;
import com.cleanroommc.modularui.utils.Vector3f;
import com.cleanroommc.modularui.widget.sizer.Area;

import org.jetbrains.annotations.ApiStatus;
import org.jetbrains.annotations.Nullable;

/**
 * A single matrix in a matrix stack. Also has some other information.
 */
@ApiStatus.Internal
public class TransformationMatrix {

    public static final TransformationMatrix EMPTY = new TransformationMatrix(null);

    private TransformationMatrix wrapped;
    private IViewport viewport;
    private Area area;
    private final Matrix4f matrix = new Matrix4f();
    private final Matrix4f invertedMatrix = new Matrix4f();
    private boolean inUse = false;

    public TransformationMatrix() {}

    public TransformationMatrix(@Nullable Matrix4f parent) {
        construct(parent);
    }

    public TransformationMatrix(TransformationMatrix parent, @Nullable Matrix4f parentMatrix) {
        construct(parent, parentMatrix);
    }

    private boolean viewportMatrix;
    private boolean dirty = true;

    TransformationMatrix construct(TransformationMatrix parent, @Nullable Matrix4f parentMatrix) {
        checkInUse();
        this.wrapped = parent;
        this.viewport = parent.viewport;
        this.area = parent.area;
        if (parentMatrix == null) {
            this.matrix.load(parent.getMatrix());
        } else {
            Matrix4f.mul(parentMatrix, parent.getMatrix(), this.matrix);
        }
        this.viewportMatrix = parent.viewportMatrix;
        return this;
    }

    TransformationMatrix construct(@Nullable Matrix4f parent) {
        return construct(null, null, parent, false);
    }

    TransformationMatrix construct(IViewport viewport, Area area, @Nullable Matrix4f parent) {
        return construct(viewport, area, parent, true);
    }

    private TransformationMatrix construct(IViewport viewport, Area area, @Nullable Matrix4f parent, boolean isViewport) {
        checkInUse();
        this.wrapped = null;
        this.viewport = viewport;
        this.area = area;
        this.viewportMatrix = isViewport;
        if (parent != null) {
            this.matrix.load(parent);
        } else {
            this.matrix.setIdentity();
        }
        return this;
    }

    void dispose() {
        this.wrapped = null;
        this.viewport = null;
        this.area = null;
        this.inUse = false;
        this.viewportMatrix = false;
    }

    private void checkInUse() {
        if (this.inUse) {
            throw new IllegalStateException("Transformation matrix is already in use!");
        }
        this.inUse = true;
        this.dirty = true;
    }

    public boolean isInUse() {
        return inUse;
    }

    public TransformationMatrix getWrapped() {
        return this.wrapped;
    }

    public IViewport getViewport() {
        return this.viewport;
    }

    public Area getArea() {
        return this.area;
    }

    public Matrix4f getMatrix() {
        return this.matrix;
    }

    public Matrix4f getInvertedMatrix() {
        if (this.dirty) {
            if (Matrix4f.invert(this.matrix, this.invertedMatrix) == null) {
                this.invertedMatrix.load(this.matrix);
            }
            this.dirty = false;
        }
        return this.invertedMatrix;
    }

    public boolean isViewportMatrix() {
        return this.viewportMatrix;
    }

    public void markDirty() {
        this.dirty = true;
    }

    public boolean isDirty() {
        return this.dirty;
    }

    public int transformX(float x, float y) {
        Matrix4f m = getMatrix();
        return (int) (x * m.m00 + y * m.m10 + m.m30);
    }

    public int transformY(float x, float y) {
        Matrix4f m = getMatrix();
        return (int) (x * m.m01 + y * m.m11 + m.m31);
    }

    public int unTransformX(float x, float y) {
        Matrix4f m = getInvertedMatrix();
        return (int) (x * m.m00 + y * m.m10 + m.m30);
    }

    public int unTransformY(float x, float y) {
        Matrix4f m = getInvertedMatrix();
        return (int) (x * m.m01 + y * m.m11 + m.m31);
    }

    public Vector3f transform(Vector3f vec, Vector3f dest) {
        return transform(getMatrix(), vec, dest);
    }

    public Vector3f unTransform(Vector3f vec, Vector3f dest) {
        return transform(getInvertedMatrix(), vec, dest);
    }

    public static Vector3f transform(Matrix4f m, Vector3f vec, Vector3f dest) {
        float x = m.m00 * vec.x + m.m10 * vec.y + m.m20 * vec.z + m.m30;
        float y = m.m01 * vec.x + m.m11 * vec.y + m.m21 * vec.z + m.m31;
        float z = m.m02 * vec.x + m.m12 * vec.y + m.m22 * vec.z + m.m32;
        dest.set(x, y, z);
        return dest;
    }
}

==> modularui-3.0.5-sources.jar: com/cleanroommc/modularui/screen/viewport/TransformationMatrix.java <==
package com.cleanroommc.modularui.screen.viewport;

import com.cleanroommc.modularui.api.layout.IViewport;
import com.cleanroommc.modularui.widget.sizer.Area;

import org.jetbrains.annotations.ApiStatus;
import org.jetbrains.annotations.Nullable;
import org.joml.Matrix4f;
import org.joml.Vector3f;

/**
 * A single matrix in a matrix stack. Also has some other information.
 */
@ApiStatus.Internal
public class TransformationMatrix {

    public static final TransformationMatrix EMPTY = new TransformationMatrix(null);

    private TransformationMatrix wrapped;
    private IViewport viewport;
    private Area area;
    private final Matrix4f matrix = new Matrix4f();
    private final Matrix4f invertedMatrix = new Matrix4f();
    private boolean inUse = false;

    public TransformationMatrix() {}

    public TransformationMatrix(@Nullable Matrix4f parent) {
        construct(parent);
    }

    public TransformationMatrix(TransformationMatrix parent, @Nullable Matrix4f parentMatrix) {
        construct(parent, parentMatrix);
    }

    private boolean viewportMatrix;
    private boolean dirty = true;

    TransformationMatrix construct(TransformationMatrix parent, @Nullable Matrix4f parentMatrix) {
        checkInUse();
        this.wrapped = parent;
        this.viewport = parent.viewport;
        this.area = parent.area;
        if (parentMatrix == null) {
            this.matrix.set(parent.getMatrix());
        } else {
            parentMatrix.mul(parent.getMatrix(), this.matrix);
        }
        this.viewportMatrix = parent.viewportMatrix;
        return this;
    }

    TransformationMatrix construct(@Nullable Matrix4f parent) {
        return construct(null, null, parent, false);
    }

    TransformationMatrix construct(IViewport viewport, Area area, @Nullable Matrix4f parent) {
        return construct(viewport, area, parent, true);
    }

    private TransformationMatrix construct(IViewport viewport, Area area, @Nullable Matrix4f parent, boolean isViewport) {
        checkInUse();
        this.wrapped = null;
        this.viewport = viewport;
        this.area = area;
        this.viewportMatrix = isViewport;
        if (parent != null) {
            this.matrix.set(parent);
        } else {
            this.matrix.identity();
        }
        return this;
    }

    void dispose() {
        this.wrapped = null;
        this.viewport = null;
        this.area = null;
        this.inUse = false;
        this.viewportMatrix = false;
    }

    private void checkInUse() {
        if (this.inUse) {
            throw new IllegalStateException("Transformation matrix is already in use!");
        }
        this.inUse = true;
        this.dirty = true;
    }

    public boolean isInUse() {
        return inUse;
    }

    public TransformationMatrix getWrapped() {
        return this.wrapped;
    }

    public IViewport getViewport() {
        return this.viewport;
    }

    public Area getArea() {
        return this.area;
    }

    public Matrix4f getMatrix() {
        return this.matrix;
    }

    public Matrix4f getInvertedMatrix() {
        if (this.dirty) {
            this.matrix.invert(this.invertedMatrix);
            this.dirty = false;
        }
        return this.invertedMatrix;
    }

    public boolean isViewportMatrix() {
        return this.viewportMatrix;
    }

    public void markDirty() {
        this.dirty = true;
    }

    public boolean isDirty() {
        return this.dirty;
    }

    public int transformX(float x, float y) {
        Matrix4f m = getMatrix();
        return (int) (x * m.m00() + y * m.m10() + m.m30());
    }

    public int transformY(float x, float y) {
        Matrix4f m = getMatrix();
        return (int) (x * m.m01() + y * m.m11() + m.m31());
    }

    public int unTransformX(float x, float y) {
        Matrix4f m = getInvertedMatrix();
        return (int) (x * m.m00() + y * m.m10() + m.m30());
    }

    public int unTransformY(float x, float y) {
        Matrix4f m = getInvertedMatrix();
        return (int) (x * m.m01() + y * m.m11() + m.m31());
    }

    public Vector3f transform(Vector3f vec, Vector3f dest) {
        return transform(getMatrix(), vec, dest);
    }

    public Vector3f unTransform(Vector3f vec, Vector3f dest) {
        return transform(getInvertedMatrix(), vec, dest);
    }

    public static Vector3f transform(Matrix4f m, Vector3f vec, Vector3f dest) {
        float x = m.m00() * vec.x + m.m10() * vec.y + m.m20() * vec.z + m.m30();
        float y = m.m01() * vec.x + m.m11() * vec.y + m.m21() * vec.z + m.m31();
        float z = m.m02() * vec.x + m.m12() * vec.y + m.m22() * vec.z + m.m32();
        dest.set(x, y, z);
        return dest;
    }
}
